@@ -34,12 +34,10 @@
 #include <pango/pango.h>
 #include <atk/atk.h>
 
-#include <clutter/clutter-types.h>
+#include <cogl/cogl.h>
 
-#include <clutter/clutter-color.h>
-#include <clutter/clutter-effect.h>
+#include <clutter/clutter-types.h>
 #include <clutter/clutter-event.h>
-#include <clutter/clutter-shader.h>
 
 G_BEGIN_DECLS
 
@@ -175,9 +173,11 @@ struct _ClutterActor
  * @unrealize: virtual function, used to deallocate resources allocated
  *   in ::realize; it should chain up to the parent's implementation
  * @map: virtual function for containers and composite actors, to
- *   map their children; it must chain up to the parent's implementation
+ *   map their children; it must chain up to the parent's implementation.
+ *   Overriding this function is optional.
  * @unmap: virtual function for containers and composite actors, to
- *   unmap their children; it must chain up to the parent's implementation
+ *   unmap their children; it must chain up to the parent's implementation.
+ *   Overriding this function is optional.
  * @paint: virtual function, used to paint the actor
  * @get_preferred_width: virtual function, used when querying the minimum
  *   and natural widths of an actor for a given height; it is used by
