@@ -98,18 +98,19 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
 
+#include "clutter-actor.h"
+#include "clutter-backend-private.h"
+#include "clutter-debug.h"
+#include "clutter-device-manager-private.h"
 #include "clutter-event.h"
-#include "clutter-backend.h"
+#include "clutter-feature.h"
+#include "clutter-frame-source.h"
 #include "clutter-main.h"
 #include "clutter-master-clock.h"
-#include "clutter-feature.h"
-#include "clutter-actor.h"
-#include "clutter-stage.h"
 #include "clutter-private.h"
-#include "clutter-debug.h"
-#include "clutter-version.h" 	/* For flavour define */
-#include "clutter-frame-source.h"
 #include "clutter-profile.h"
+#include "clutter-stage-private.h"
+#include "clutter-version.h" 	/* For flavour define */
 
 #include "cogl/cogl.h"
 #include "pango/cogl-pango.h"
@@ -134,7 +135,7 @@ static gboolean clutter_enable_accessibility = TRUE;
 
 static guint clutter_default_fps             = 60;
 
-static PangoDirection clutter_text_direction = CLUTTER_TEXT_DIRECTION_LTR;
+static ClutterTextDirection clutter_text_direction = CLUTTER_TEXT_DIRECTION_LTR;
 
 static guint clutter_main_loop_level         = 0;
 static GSList *main_loops                    = NULL;
@@ -179,7 +180,9 @@ static const GDebugKey clutter_pick_debug_keys[] = {
 static const GDebugKey clutter_paint_debug_keys[] = {
   { "disable-swap-events", CLUTTER_DEBUG_DISABLE_SWAP_EVENTS },
   { "disable-clipped-redraws", CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS },
-  { "redraws", CLUTTER_DEBUG_REDRAWS }
+  { "redraws", CLUTTER_DEBUG_REDRAWS },
+  { "paint-volumes", CLUTTER_DEBUG_PAINT_VOLUMES },
+  { "disable-culling", CLUTTER_DEBUG_DISABLE_CULLING }
 };
 
 #ifdef CLUTTER_ENABLE_PROFILE
@@ -224,47 +227,6 @@ clutter_get_accessibility_enabled (void)
   return cally_get_cally_initialized ();
 }
 
-
-void
-_clutter_stage_maybe_relayout (ClutterActor *stage)
-{
-  gfloat natural_width, natural_height;
-  ClutterActorBox box = { 0, };
-  CLUTTER_STATIC_TIMER (relayout_timer,
-                        "Mainloop", /* no parent */
-                        "Layouting",
-                        "The time spent reallocating the stage",
-                        0 /* no application private data */);
-
-  /* avoid reentrancy */
-  if (!CLUTTER_ACTOR_IN_RELAYOUT (stage))
-    {
-      CLUTTER_TIMER_START (_clutter_uprof_context, relayout_timer);
-      CLUTTER_NOTE (ACTOR, "Recomputing layout");
-
-      CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_IN_RELAYOUT);
-
-      natural_width = natural_height = 0;
-      clutter_actor_get_preferred_size (stage,
-                                        NULL, NULL,
-                                        &natural_width, &natural_height);
-
-      box.x1 = 0;
-      box.y1 = 0;
-      box.x2 = natural_width;
-      box.y2 = natural_height;
-
-      CLUTTER_NOTE (ACTOR, "Allocating (0, 0 - %d, %d) for the stage",
-                    (int) natural_width,
-                    (int) natural_height);
-
-      clutter_actor_allocate (stage, &box, CLUTTER_ALLOCATION_NONE);
-
-      CLUTTER_UNSET_PRIVATE_FLAGS (stage, CLUTTER_IN_RELAYOUT);
-      CLUTTER_TIMER_STOP (_clutter_uprof_context, relayout_timer);
-    }
-}
-
 void
 _clutter_do_redraw (ClutterStage *stage)
 {
@@ -274,15 +236,8 @@ _clutter_do_redraw (ClutterStage *stage)
 
   ctx = _clutter_context_get_default ();
 
-  /* We have an optimization to avoid pick renders while scene is static.
-   * This is implied by seeing multiple picks per frame so we have to
-   * reset a pick counter each frame and also invalidate any current pick
-   * buffer. */
-  ctx->picks_per_frame = 0;
-  ctx->have_complete_pick_buffer = FALSE;
-
-  /* Before we can paint, we have to be sure we have the latest layout */
-  _clutter_stage_maybe_relayout (CLUTTER_ACTOR (stage));
+  _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
+  _clutter_stage_reset_picks_per_frame_counter (stage);
 
   _clutter_backend_ensure_context (ctx->backend, stage);
 
@@ -389,6 +344,26 @@ _clutter_id_to_color (guint id, ClutterColor *col)
 
   ctx = _clutter_context_get_default ();
 
+  if (ctx->fb_g_mask == 0)
+    {
+      /* Figure out framebuffer masks used for pick */
+      cogl_get_bitmasks (&ctx->fb_r_mask,
+			 &ctx->fb_g_mask,
+			 &ctx->fb_b_mask, NULL);
+
+      ctx->fb_r_mask_used = ctx->fb_r_mask;
+      ctx->fb_g_mask_used = ctx->fb_g_mask;
+      ctx->fb_b_mask_used = ctx->fb_b_mask;
+
+      /* XXX - describe what "fuzzy picking" is */
+      if (clutter_use_fuzzy_picking)
+	{
+	  ctx->fb_r_mask_used--;
+	  ctx->fb_g_mask_used--;
+	  ctx->fb_b_mask_used--;
+	}
+    }
+
   /* compute the numbers we'll store in the components */
   red   = (id >> (ctx->fb_g_mask_used+ctx->fb_b_mask_used))
         & (0xff >> (8-ctx->fb_r_mask_used));
@@ -601,7 +576,7 @@ _clutter_do_pick (ClutterStage   *stage,
   /* It's possible that we currently have a static scene and have renderered a
    * full, unclipped pick buffer. If so we can simply continue to read from
    * this cached buffer until the scene next changes. */
-  if (context->have_complete_pick_buffer)
+  if (_clutter_stage_get_pick_buffer_valid (stage, mode))
     {
       CLUTTER_TIMER_START (_clutter_uprof_context, pick_read);
       cogl_read_pixels (x, y, 1, 1,
@@ -628,7 +603,7 @@ _clutter_do_pick (ClutterStage   *stage,
       goto result;
     }
 
-  context->picks_per_frame++;
+  _clutter_stage_increment_picks_per_frame_counter (stage);
 
   _clutter_backend_ensure_context (context->backend, stage);
 
@@ -638,7 +613,7 @@ _clutter_do_pick (ClutterStage   *stage,
   /* If we are seeing multiple picks per frame that means the scene is static
    * so we promote to doing a non-scissored pick render so that all subsequent
    * picks for the same static scene won't require additional renders */
-  if (context->picks_per_frame < 2)
+  if (_clutter_stage_get_picks_per_frame_counter (stage) < 2)
     {
       if (G_LIKELY (!(clutter_pick_debug_flags &
                       CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
@@ -669,7 +644,7 @@ _clutter_do_pick (ClutterStage   *stage,
   */
   CLUTTER_TIMER_START (_clutter_uprof_context, pick_paint);
   context->pick_mode = mode;
-  clutter_actor_paint (CLUTTER_ACTOR (stage));
+  _clutter_stage_do_paint (stage, NULL);
   context->pick_mode = CLUTTER_PICK_NONE;
   CLUTTER_TIMER_STOP (_clutter_uprof_context, pick_paint);
 
@@ -678,9 +653,11 @@ _clutter_do_pick (ClutterStage   *stage,
       if (G_LIKELY (!(clutter_pick_debug_flags &
                       CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
         cogl_clip_pop ();
+
+      _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
     }
   else
-    context->have_complete_pick_buffer = TRUE;
+    _clutter_stage_set_pick_buffer_valid (stage, TRUE, mode);
 
   /* Make sure Cogl flushes any batched geometry to the GPU driver */
   cogl_flush ();
@@ -1071,15 +1048,8 @@ clutter_threads_set_lock_functions (GCallback enter_fn,
   clutter_threads_unlock = leave_fn;
 }
 
-typedef struct
-{
-  GSourceFunc func;
-  gpointer data;
-  GDestroyNotify notify;
-} ClutterThreadsDispatch;
-
-static gboolean
-clutter_threads_dispatch (gpointer data)
+gboolean
+_clutter_threads_dispatch (gpointer data)
 {
   ClutterThreadsDispatch *dispatch = data;
   gboolean ret = FALSE;
@@ -1094,8 +1064,8 @@ clutter_threads_dispatch (gpointer data)
   return ret;
 }
 
-static void
-clutter_threads_dispatch_free (gpointer data)
+void
+_clutter_threads_dispatch_free (gpointer data)
 {
   ClutterThreadsDispatch *dispatch = data;
 
@@ -1222,8 +1192,8 @@ clutter_threads_add_idle_full (gint           priority,
   dispatch->notify = notify;
 
   return g_idle_add_full (priority,
-                          clutter_threads_dispatch, dispatch,
-                          clutter_threads_dispatch_free);
+                          _clutter_threads_dispatch, dispatch,
+                          _clutter_threads_dispatch_free);
 }
 
 /**
@@ -1298,8 +1268,8 @@ clutter_threads_add_timeout_full (gint           priority,
 
   return g_timeout_add_full (priority,
                              interval,
-                             clutter_threads_dispatch, dispatch,
-                             clutter_threads_dispatch_free);
+                             _clutter_threads_dispatch, dispatch,
+                             _clutter_threads_dispatch_free);
 }
 
 /**
@@ -1327,86 +1297,6 @@ clutter_threads_add_timeout (guint       interval,
                                            NULL);
 }
 
-/**
- * clutter_threads_add_frame_source_full:
- * @priority: the priority of the frame source. Typically this will be in the
- *            range between #G_PRIORITY_DEFAULT and #G_PRIORITY_HIGH.
- * @fps: the number of times per second to call the function
- * @func: function to call
- * @data: data to pass to the function
- * @notify: function to call when the timeout source is removed
- *
- * Sets a function to be called at regular intervals holding the Clutter
- * threads lock, with the given priority. The function is called repeatedly
- * until it returns %FALSE, at which point the timeout is automatically
- * removed and the function will not be called again. The @notify function
- * is called when the timeout is removed.
- *
- * This function is similar to clutter_threads_add_timeout_full()
- * except that it will try to compensate for delays. For example, if
- * @func takes half the interval time to execute then the function
- * will be called again half the interval time after it finished. In
- * contrast clutter_threads_add_timeout_full() would not fire until a
- * full interval after the function completes so the delay between
- * calls would be @interval * 1.5. This function does not however try
- * to invoke the function multiple times to catch up missing frames if
- * @func takes more than @interval ms to execute.
- *
- * See also clutter_threads_add_idle_full().
- *
- * Rename to: clutter_threads_add_frame_source
- *
- * Return value: the ID (greater than 0) of the event source.
- *
- * Since: 0.8
- */
-guint
-clutter_threads_add_frame_source_full (gint           priority,
-				       guint          fps,
-				       GSourceFunc    func,
-				       gpointer       data,
-				       GDestroyNotify notify)
-{
-  ClutterThreadsDispatch *dispatch;
-
-  g_return_val_if_fail (func != NULL, 0);
-
-  dispatch = g_slice_new (ClutterThreadsDispatch);
-  dispatch->func = func;
-  dispatch->data = data;
-  dispatch->notify = notify;
-
-  return clutter_frame_source_add_full (priority,
-					fps,
-					clutter_threads_dispatch, dispatch,
-					clutter_threads_dispatch_free);
-}
-
-/**
- * clutter_threads_add_frame_source: (skip)
- * @fps: the number of times per second to call the function
- * @func: function to call
- * @data: data to pass to the function
- *
- * Simple wrapper around clutter_threads_add_frame_source_full().
- *
- * Return value: the ID (greater than 0) of the event source.
- *
- * Since: 0.8
- */
-guint
-clutter_threads_add_frame_source (guint       fps,
-				  GSourceFunc func,
-				  gpointer    data)
-{
-  g_return_val_if_fail (func != NULL, 0);
-
-  return clutter_threads_add_frame_source_full (G_PRIORITY_DEFAULT,
-						fps,
-						func, data,
-						NULL);
-}
-
 /**
  * clutter_threads_enter:
  *
@@ -1614,6 +1504,17 @@ clutter_init_real (GError **error)
   if (!_clutter_backend_post_parse (backend, error))
     return CLUTTER_INIT_ERROR_BACKEND;
 
+  /* If we are displaying the regions that would get redrawn with clipped
+   * redraws enabled we actually have to disable the clipped redrawing
+   * because otherwise we end up with nasty trails of rectangles everywhere.
+   */
+  if (clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS)
+    clutter_paint_debug_flags |= CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS;
+
+  /* The same is true when drawing the outlines of paint volumes... */
+  if (clutter_paint_debug_flags & CLUTTER_DEBUG_PAINT_VOLUMES)
+    clutter_paint_debug_flags |= CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS;
+
   /* this will take care of initializing Cogl's state and
    * query the GL machinery for features
    */
@@ -1641,21 +1542,6 @@ clutter_init_real (GError **error)
 
   clutter_text_direction = clutter_get_text_direction ();
 
-  /* Figure out framebuffer masks used for pick */
-  cogl_get_bitmasks (&ctx->fb_r_mask, &ctx->fb_g_mask, &ctx->fb_b_mask, NULL);
-
-  ctx->fb_r_mask_used = ctx->fb_r_mask;
-  ctx->fb_g_mask_used = ctx->fb_g_mask;
-  ctx->fb_b_mask_used = ctx->fb_b_mask;
-
-  /* XXX - describe what "fuzzy picking" is */
-  if (clutter_use_fuzzy_picking)
-    {
-      ctx->fb_r_mask_used--;
-      ctx->fb_g_mask_used--;
-      ctx->fb_b_mask_used--;
-    }
-
   /* Initiate event collection */
   _clutter_backend_init_events (ctx->backend);
 
