@@ -46,25 +46,18 @@ cogl_clip_push_window_rectangle (int x_offset,
                                  int width,
                                  int height)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   CoglClipState *clip_state;
-  CoglHandle stack;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
-
   framebuffer = _cogl_get_framebuffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
-  stack = clip_state->stacks->data;
-
-  _cogl_clip_stack_push_window_rectangle (stack, x_offset, y_offset,
-                                          width, height);
-
-  clip_state->stack_dirty = TRUE;
+  clip_state->stacks->data =
+    _cogl_clip_stack_push_window_rectangle (clip_state->stacks->data,
+                                            x_offset, y_offset,
+                                            width, height);
 }
 
 /* XXX: This is deprecated API */
@@ -133,17 +126,12 @@ cogl_clip_push_rectangle (float x_1,
                           float x_2,
                           float y_2)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   CoglClipState *clip_state;
-  CoglHandle stack;
   CoglMatrix modelview_matrix;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
-
   /* Try and catch window space rectangles so we can redirect to
    * cogl_clip_push_window_rect which will use scissoring. */
   if (try_pushing_rect_as_window_rect (x_1, y_1, x_2, y_2))
@@ -152,14 +140,12 @@ cogl_clip_push_rectangle (float x_1,
   framebuffer = _cogl_get_framebuffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
-  stack = clip_state->stacks->data;
-
   cogl_get_modelview_matrix (&modelview_matrix);
 
-  _cogl_clip_stack_push_rectangle (stack, x_1, y_1, x_2, y_2,
-                                   &modelview_matrix);
-
-  clip_state->stack_dirty = TRUE;
+  clip_state->stacks->data =
+    _cogl_clip_stack_push_rectangle (clip_state->stacks->data,
+                                     x_1, y_1, x_2, y_2,
+                                     &modelview_matrix);
 }
 
 /* XXX: Deprecated API */
@@ -178,58 +164,44 @@ cogl_clip_push (float x_offset,
 void
 cogl_clip_push_from_path_preserve (void)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   CoglClipState *clip_state;
-  CoglHandle stack;
   CoglMatrix modelview_matrix;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
-
   framebuffer = _cogl_get_framebuffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
-  stack = clip_state->stacks->data;
-
   cogl_get_modelview_matrix (&modelview_matrix);
 
-  _cogl_clip_stack_push_from_path (stack, cogl_get_path (),
-                                   &modelview_matrix);
-
-  clip_state->stack_dirty = TRUE;
+  clip_state->stacks->data =
+    _cogl_clip_stack_push_from_path (clip_state->stacks->data,
+                                     ctx->current_path,
+                                     &modelview_matrix);
 }
 
 void
 cogl_clip_push_from_path (void)
 {
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   cogl_clip_push_from_path_preserve ();
 
-  cogl_path_new ();
+  cogl_object_unref (ctx->current_path);
+  ctx->current_path = cogl2_path_new ();
 }
 
 static void
 _cogl_clip_pop_real (CoglClipState *clip_state)
 {
-  CoglHandle stack;
-
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
-
-  stack = clip_state->stacks->data;
-
-  _cogl_clip_stack_pop (stack);
-
-  clip_state->stack_dirty = TRUE;
+  clip_state->stacks->data = _cogl_clip_stack_pop (clip_state->stacks->data);
 }
 
 void
 cogl_clip_pop (void)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   CoglClipState *clip_state;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -243,25 +215,9 @@ cogl_clip_pop (void)
 void
 _cogl_clip_state_flush (CoglClipState *clip_state)
 {
-  CoglHandle stack;
-
-  if (!clip_state->stack_dirty)
-    return;
-
-  /* The current primitive journal does not support tracking changes to the
-   * clip stack...  */
-  _cogl_journal_flush ();
-
-  /* XXX: the handling of clipping is quite complex. It may involve use of
-   * the Cogl Journal or other Cogl APIs which may end up recursively
-   * wanting to ensure the clip state is flushed. We need to ensure we
-   * don't recurse infinitely...
-   */
-  clip_state->stack_dirty = FALSE;
-
-  stack = clip_state->stacks->data;
-
-  _cogl_clip_stack_flush (stack, &clip_state->stencil_used);
+  /* Flush the topmost stack. The clip stack code will bail out early
+     if this is already flushed */
+  _cogl_clip_stack_flush (clip_state->stacks->data);
 }
 
 /* XXX: This should never have been made public API! */
@@ -271,28 +227,24 @@ cogl_clip_ensure (void)
   CoglClipState *clip_state;
 
   clip_state = _cogl_framebuffer_get_clip_state (_cogl_get_framebuffer ());
+  /* Flushing the clip state doesn't cause the journal to be
+     flushed. This function may be being called by an external
+     application however so it makes sense to flush the journal
+     here */
+  _cogl_journal_flush ();
   _cogl_clip_state_flush (clip_state);
 }
 
 static void
 _cogl_clip_stack_save_real (CoglClipState *clip_state)
 {
-  CoglHandle stack;
-
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
-
-  stack = _cogl_clip_stack_new ();
-
-  clip_state->stacks = g_slist_prepend (clip_state->stacks, stack);
-  clip_state->stack_dirty = TRUE;
+  clip_state->stacks = g_slist_prepend (clip_state->stacks, NULL);
 }
 
 void
 cogl_clip_stack_save (void)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   CoglClipState *clip_state;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -310,25 +262,19 @@ _cogl_clip_stack_restore_real (CoglClipState *clip_state)
 
   g_return_if_fail (clip_state->stacks != NULL);
 
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
-
   stack = clip_state->stacks->data;
 
-  cogl_handle_unref (stack);
+  _cogl_clip_stack_unref (stack);
 
   /* Revert to an old stack */
   clip_state->stacks = g_slist_delete_link (clip_state->stacks,
                                             clip_state->stacks);
-
-  clip_state->stack_dirty = TRUE;
 }
 
 void
 cogl_clip_stack_restore (void)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   CoglClipState *clip_state;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -345,7 +291,6 @@ _cogl_clip_state_init (CoglClipState *clip_state)
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   clip_state->stacks = NULL;
-  clip_state->stack_dirty = TRUE;
 
   /* Add an intial stack */
   _cogl_clip_stack_save_real (clip_state);
@@ -359,16 +304,10 @@ _cogl_clip_state_destroy (CoglClipState *clip_state)
     _cogl_clip_stack_restore_real (clip_state);
 }
 
-void
-_cogl_clip_state_dirty (CoglClipState *clip_state)
-{
-  clip_state->stack_dirty = TRUE;
-}
-
-CoglHandle
+CoglClipStack *
 _cogl_get_clip_stack (void)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   CoglClipState *clip_state;
 
   _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
@@ -380,21 +319,18 @@ _cogl_get_clip_stack (void)
 }
 
 void
-_cogl_set_clip_stack (CoglHandle handle)
+_cogl_set_clip_stack (CoglClipStack *stack)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   CoglClipState *clip_state;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (handle == COGL_INVALID_HANDLE)
-    return;
-
   framebuffer = _cogl_get_framebuffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
   /* Replace the top of the stack of stacks */
-  cogl_handle_ref (handle);
-  cogl_handle_unref (clip_state->stacks->data);
-  clip_state->stacks->data = handle;
+  _cogl_clip_stack_ref (stack);
+  _cogl_clip_stack_unref (clip_state->stacks->data);
+  clip_state->stacks->data = stack;
 }
