@@ -27,14 +27,15 @@
 
 #include "cogl.h"
 #include "cogl-internal.h"
+#include "cogl-profile.h"
 #include "cogl-util.h"
 #include "cogl-context.h"
 #include "cogl-journal-private.h"
 #include "cogl-texture-private.h"
-#include "cogl-material-private.h"
-#include "cogl-material-opengl-private.h"
+#include "cogl-pipeline-private.h"
+#include "cogl-pipeline-opengl-private.h"
 #include "cogl-framebuffer-private.h"
-#include "cogl-path-private.h"
+#include "cogl2-path.h"
 
 #include <string.h>
 
@@ -57,6 +58,28 @@ _cogl_destroy_context_winsys (CoglContext *context);
 static CoglContext *_context = NULL;
 static gboolean gl_is_indirect = FALSE;
 
+static void
+_cogl_init_feature_overrides (CoglContext *ctx)
+{
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS))
+    ctx->feature_flags &= ~COGL_FEATURE_VBOS;
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_PBOS))
+    ctx->feature_flags &= ~COGL_FEATURE_PBOS;
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_ARBFP))
+    ctx->feature_flags &= ~COGL_FEATURE_SHADERS_ARBFP;
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_GLSL))
+    ctx->feature_flags &= ~COGL_FEATURE_SHADERS_GLSL;
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_NPOT_TEXTURES))
+    ctx->feature_flags &= ~(COGL_FEATURE_TEXTURE_NPOT |
+                            COGL_FEATURE_TEXTURE_NPOT_BASIC |
+                            COGL_FEATURE_TEXTURE_NPOT_MIPMAP |
+                            COGL_FEATURE_TEXTURE_NPOT_REPEAT);
+}
+
 static gboolean
 cogl_create_context (void)
 {
@@ -68,13 +91,28 @@ cogl_create_context (void)
   if (_context != NULL)
     return FALSE;
 
+#ifdef CLUTTER_ENABLE_PROFILE
+  /* We need to be absolutely sure that uprof has been initialized
+   * before calling _cogl_uprof_init. uprof_init (NULL, NULL)
+   * will be a NOP if it has been initialized but it will also
+   * mean subsequent parsing of the UProf GOptionGroup will have no
+   * affect.
+   *
+   * Sadly GOptionGroup based library initialization is extremely
+   * fragile by design because GOptionGroups have no notion of
+   * dependencies and so the order things are initialized isn't
+   * currently under tight control.
+   */
+  uprof_init (NULL, NULL);
+  _cogl_uprof_init ();
+#endif
+
   /* Allocate context memory */
   _context = (CoglContext*) g_malloc (sizeof (CoglContext));
 
   /* Init default values */
   _context->feature_flags = 0;
   _context->feature_flags_private = 0;
-  _context->features_cached = FALSE;
 
   _context->texture_types = NULL;
   _context->buffer_types = NULL;
@@ -83,11 +121,12 @@ cogl_create_context (void)
   /* TODO: combine these two into one function */
   _cogl_create_context_driver (_context);
   _cogl_features_init ();
+  _cogl_init_feature_overrides (_context);
 
   _cogl_create_context_winsys (_context);
 
-  _cogl_material_init_default_material ();
-  _cogl_material_init_default_layers ();
+  _cogl_pipeline_init_default_pipeline ();
+  _cogl_pipeline_init_default_layers ();
 
   _context->enable_flags = 0;
 
@@ -105,16 +144,19 @@ cogl_create_context (void)
   _context->texture_units =
     g_array_new (FALSE, FALSE, sizeof (CoglTextureUnit));
 
-  /* See cogl-material.c for more details about why we leave texture unit 1
+  /* See cogl-pipeline.c for more details about why we leave texture unit 1
    * active by default... */
   _context->active_texture_unit = 1;
   GE (glActiveTexture (GL_TEXTURE1));
 
   _context->legacy_fog_state.enabled = FALSE;
 
-  _context->simple_material = cogl_material_new ();
-  _context->source_material = NULL;
-  _context->arbfp_source_buffer = g_string_new ("");
+  _context->opaque_color_pipeline = cogl_pipeline_new ();
+  _context->blended_color_pipeline = cogl_pipeline_new ();
+  _context->texture_pipeline = cogl_pipeline_new ();
+  _context->fragment_header_buffer = g_string_new ("");
+  _context->fragment_source_buffer = g_string_new ("");
+  _context->source_stack = NULL;
 
   _context->legacy_state_set = 0;
 
@@ -123,14 +165,18 @@ cogl_create_context (void)
 
   _context->journal = g_array_new (FALSE, FALSE, sizeof (CoglJournalEntry));
   _context->logged_vertices = g_array_new (FALSE, FALSE, sizeof (GLfloat));
+  _context->journal_flush_attributes_array =
+    g_array_new (TRUE, FALSE, sizeof (CoglVertexAttribute *));
 
-  _context->current_material = NULL;
-  _context->current_material_changes_since_flush = 0;
-  _context->current_material_skip_gl_color = FALSE;
+  _context->polygon_vertices = g_array_new (FALSE, FALSE, sizeof (float));
 
-  _context->material0_nodes =
+  _context->current_pipeline = NULL;
+  _context->current_pipeline_changes_since_flush = 0;
+  _context->current_pipeline_skip_gl_color = FALSE;
+
+  _context->pipeline0_nodes =
     g_array_sized_new (FALSE, FALSE, sizeof (CoglHandle), 20);
-  _context->material1_nodes =
+  _context->pipeline1_nodes =
     g_array_sized_new (FALSE, FALSE, sizeof (CoglHandle), 20);
 
   _cogl_bitmask_init (&_context->texcoord_arrays_enabled);
@@ -143,7 +189,7 @@ cogl_create_context (void)
 
   _context->current_program = COGL_INVALID_HANDLE;
 
-  _context->current_use_program_type = COGL_MATERIAL_PROGRAM_TYPE_FIXED;
+  _context->current_use_program_type = COGL_PIPELINE_PROGRAM_TYPE_FIXED;
   _context->current_gl_program = 0;
 
   _context->gl_blend_enable_cache = FALSE;
@@ -163,6 +209,8 @@ cogl_create_context (void)
 
   _context->framebuffer_stack = _cogl_create_framebuffer_stack ();
 
+  _context->current_clip_stack_valid = FALSE;
+
   window_buffer = _cogl_onscreen_new ();
   cogl_set_framebuffer (window_buffer);
   /* XXX: the deprecated _cogl_set_draw_buffer API expects to
@@ -172,16 +220,20 @@ cogl_create_context (void)
   _context->dirty_bound_framebuffer = TRUE;
   _context->dirty_gl_viewport = TRUE;
 
-  _context->current_path = _cogl_path_new ();
-  _context->stencil_material = cogl_material_new ();
+  _context->current_path = cogl2_path_new ();
+  _context->stencil_pipeline = cogl_pipeline_new ();
 
   _context->in_begin_gl_block = FALSE;
 
-  _context->quad_indices_byte = COGL_INVALID_HANDLE;
-  _context->quad_indices_short = COGL_INVALID_HANDLE;
-  _context->quad_indices_short_len = 0;
+  _context->quad_buffer_indices_byte = COGL_INVALID_HANDLE;
+  _context->quad_buffer_indices = COGL_INVALID_HANDLE;
+  _context->quad_buffer_indices_len = 0;
 
-  _context->texture_download_material = COGL_INVALID_HANDLE;
+  _context->rectangle_byte_indices = NULL;
+  _context->rectangle_short_indices = NULL;
+  _context->rectangle_short_indices_len = 0;
+
+  _context->texture_download_pipeline = COGL_INVALID_HANDLE;
 
   /* The default for GL_ALPHA_TEST is to always pass which is equivalent to
    * the test being disabled therefore we assume that for all drivers there
@@ -209,19 +261,18 @@ cogl_create_context (void)
                                 0, /* auto calc row stride */
                                 default_texture_data);
 
-  cogl_set_source (_context->simple_material);
-  _cogl_material_flush_gl_state (_context->source_material, FALSE);
+  cogl_push_source (_context->opaque_color_pipeline);
+  _cogl_pipeline_flush_gl_state (_context->opaque_color_pipeline, FALSE, 0);
   _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
 
   _context->atlas = NULL;
-  _context->atlas_texture = COGL_INVALID_HANDLE;
 
   /* As far as I can tell, GL_POINT_SPRITE doesn't have any effect
      unless GL_COORD_REPLACE is enabled for an individual
      layer. Therefore it seems like it should be ok to just leave it
      enabled all the time instead of having to have a set property on
-     each material to track whether any layers have point sprite
+     each pipeline to track whether any layers have point sprite
      coords enabled */
   if (cogl_features_available (COGL_FEATURE_POINT_SPRITE))
     GE (glEnable (GL_POINT_SPRITE));
@@ -250,21 +301,35 @@ _cogl_destroy_context (void)
   if (_context->default_gl_texture_rect_tex)
     cogl_handle_unref (_context->default_gl_texture_rect_tex);
 
-  if (_context->simple_material)
-    cogl_handle_unref (_context->simple_material);
+  if (_context->opaque_color_pipeline)
+    cogl_handle_unref (_context->opaque_color_pipeline);
+  if (_context->blended_color_pipeline)
+    cogl_handle_unref (_context->blended_color_pipeline);
+  if (_context->texture_pipeline)
+    cogl_handle_unref (_context->texture_pipeline);
 
   if (_context->journal)
     g_array_free (_context->journal, TRUE);
   if (_context->logged_vertices)
     g_array_free (_context->logged_vertices, TRUE);
+  if (_context->journal_flush_attributes_array)
+    g_array_free (_context->journal_flush_attributes_array, TRUE);
 
-  if (_context->quad_indices_byte)
-    cogl_handle_unref (_context->quad_indices_byte);
-  if (_context->quad_indices_short)
-    cogl_handle_unref (_context->quad_indices_short);
+  if (_context->polygon_vertices)
+    g_array_free (_context->polygon_vertices, TRUE);
 
-  if (_context->default_material)
-    cogl_handle_unref (_context->default_material);
+  if (_context->quad_buffer_indices_byte)
+    cogl_handle_unref (_context->quad_buffer_indices_byte);
+  if (_context->quad_buffer_indices)
+    cogl_handle_unref (_context->quad_buffer_indices);
+
+  if (_context->rectangle_byte_indices)
+    cogl_object_unref (_context->rectangle_byte_indices);
+  if (_context->rectangle_short_indices)
+    cogl_object_unref (_context->rectangle_short_indices);
+
+  if (_context->default_pipeline)
+    cogl_handle_unref (_context->default_pipeline);
 
   if (_context->dummy_layer_dependant)
     cogl_handle_unref (_context->dummy_layer_dependant);
@@ -273,10 +338,11 @@ _cogl_destroy_context (void)
   if (_context->default_layer_0)
     cogl_handle_unref (_context->default_layer_0);
 
+  if (_context->current_clip_stack_valid)
+    _cogl_clip_stack_unref (_context->current_clip_stack);
+
   if (_context->atlas)
     _cogl_atlas_free (_context->atlas);
-  if (_context->atlas_texture)
-    cogl_handle_unref (_context->atlas_texture);
 
   _cogl_bitmask_destroy (&_context->texcoord_arrays_enabled);
   _cogl_bitmask_destroy (&_context->temp_bitmask);
