@@ -51,11 +51,14 @@ typedef enum {
   COGL_DEBUG_OPENGL           = 1 << 18,
   COGL_DEBUG_DISABLE_TEXTURING = 1 << 19,
   COGL_DEBUG_DISABLE_ARBFP    = 1 << 20,
-  COGL_DEBUG_DISABLE_GLSL     = 1 << 21,
-  COGL_DEBUG_SHOW_SOURCE      = 1 << 22,
-  COGL_DEBUG_DISABLE_BLENDING = 1 << 23,
-  COGL_DEBUG_TEXTURE_PIXMAP   = 1 << 24,
-  COGL_DEBUG_BITMAP           = 1 << 25
+  COGL_DEBUG_DISABLE_FIXED    = 1 << 21,
+  COGL_DEBUG_DISABLE_GLSL     = 1 << 22,
+  COGL_DEBUG_SHOW_SOURCE      = 1 << 23,
+  COGL_DEBUG_DISABLE_BLENDING = 1 << 24,
+  COGL_DEBUG_TEXTURE_PIXMAP   = 1 << 25,
+  COGL_DEBUG_BITMAP           = 1 << 26,
+  COGL_DEBUG_DISABLE_NPOT_TEXTURES = 1 << 27,
+  COGL_DEBUG_WIREFRAME        = 1 << 28
 } CoglDebugFlags;
 
 #ifdef COGL_ENABLE_DEBUG
