@@ -31,33 +31,17 @@
 #include "cogl-context.h"
 #include "cogl-journal-private.h"
 #include "cogl-texture-private.h"
-#include "cogl-material-private.h"
-#include "cogl-material-opengl-private.h"
+#include "cogl-pipeline-private.h"
+#include "cogl-pipeline-opengl-private.h"
 #include "cogl-vertex-buffer-private.h"
 #include "cogl-framebuffer-private.h"
 #include "cogl-profile.h"
+#include "cogl-vertex-attribute-private.h"
 
 #include <string.h>
 #include <gmodule.h>
 #include <math.h>
 
-#define _COGL_MAX_BEZ_RECURSE_DEPTH 16
-
-#ifdef HAVE_COGL_GL
-
-#define glGenBuffers ctx->drv.pf_glGenBuffers
-#define glBindBuffer ctx->drv.pf_glBindBuffer
-#define glBufferData ctx->drv.pf_glBufferData
-#define glBufferSubData ctx->drv.pf_glBufferSubData
-#define glDeleteBuffers ctx->drv.pf_glDeleteBuffers
-#define glClientActiveTexture ctx->drv.pf_glClientActiveTexture
-
-#elif defined (HAVE_COGL_GLES2)
-
-#include "../gles/cogl-gles2-wrapper.h"
-
-#endif
-
 /* XXX NB:
  * Our journal's vertex data is arranged as follows:
  * 4 vertices per quad:
@@ -65,7 +49,7 @@
  *    4 RGBA GLubytes,
  *    2 GLfloats per tex coord * n_layers
  *
- * Where n_layers corresponds to the number of material layers enabled
+ * Where n_layers corresponds to the number of pipeline layers enabled
  *
  * To avoid frequent changes in the stride of our vertex data we always pad
  * n_layers to be >= 2
@@ -86,21 +70,23 @@
   (POS_STRIDE + COLOR_STRIDE + \
    TEX_STRIDE * (N_LAYERS < MIN_LAYER_PADING ? MIN_LAYER_PADING : N_LAYERS))
 
-typedef CoglVertexBufferIndices  CoglJournalIndices;
-
 typedef struct _CoglJournalFlushState
 {
-  gsize              stride;
-  /* Note: this is a pointer to handle fallbacks. It normally holds a VBO
-   * offset, but when the driver doesn't support VBOs then this points into
-   * our GArray of logged vertices. */
-  char *              vbo_offset;
-  GLuint              vertex_offset;
+  CoglVertexArray     *vertex_array;
+  GArray              *attributes;
+  int                  current_attribute;
+
+  gsize                stride;
+  size_t               array_offset;
+  GLuint               current_vertex;
 #ifndef HAVE_COGL_GL
-  CoglJournalIndices *indices;
-  gsize              indices_type_size;
+  CoglIndices         *indices;
+  gsize                indices_type_size;
 #endif
-  CoglMatrixStack    *modelview_stack;
+  CoglMatrixStack     *modelview_stack;
+  CoglMatrixStack     *projection_stack;
+
+  CoglPipeline        *source;
 } CoglJournalFlushState;
 
 typedef void (*CoglJournalBatchCallback) (CoglJournalEntry *start,
@@ -197,9 +183,19 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
                                            void             *data)
 {
   CoglJournalFlushState *state = data;
+  CoglVertexAttribute **attributes;
+  COGL_STATIC_TIMER (time_flush_modelview_and_entries,
+                     "flush: pipeline+entries", /* parent */
+                     "flush: modelview+entries",
+                     "The time spent flushing modelview + entries",
+                     0 /* no application private data */);
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  COGL_TIMER_START (_cogl_uprof_context, time_flush_modelview_and_entries);
 
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_BATCHING))
-    g_print ("BATCHING:    modelview batch len = %d\n", batch_len);
+    g_print ("BATCHING:     modelview batch len = %d\n", batch_len);
 
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM))
     {
@@ -209,25 +205,30 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
                                       COGL_MATRIX_MODELVIEW);
     }
 
+  attributes = (CoglVertexAttribute **)state->attributes->data;
+  cogl_push_source (state->source);
+
 #ifdef HAVE_COGL_GL
 
-  GE (glDrawArrays (GL_QUADS, state->vertex_offset, batch_len * 4));
+  /* XXX: it's rather evil that we sneak in the GL_QUADS enum here... */
+  _cogl_draw_vertex_attributes_array (GL_QUADS,
+                                      state->current_vertex, batch_len * 4,
+                                      attributes);
 
 #else /* HAVE_COGL_GL */
-
   if (batch_len > 1)
     {
-      int indices_offset = (state->vertex_offset / 4) * 6;
-      GE (glDrawElements (GL_TRIANGLES,
-                          6 * batch_len,
-                          state->indices->type,
-                          (GLvoid*)(indices_offset * state->indices_type_size)));
+      _cogl_draw_indexed_vertex_attributes_array (COGL_VERTICES_MODE_TRIANGLES,
+                                                  state->current_vertex * 6 / 4,
+                                                  batch_len * 6,
+                                                  state->indices,
+                                                  attributes);
     }
   else
     {
-      GE (glDrawArrays (GL_TRIANGLE_FAN,
-                        state->vertex_offset, /* first */
-                        4)); /* n vertices */
+      _cogl_draw_vertex_attributes_array (COGL_VERTICES_MODE_TRIANGLE_FAN,
+                                          state->current_vertex, 4,
+                                          attributes);
     }
 #endif
 
@@ -239,14 +240,15 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
    */
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_RECTANGLES))
     {
-      static CoglHandle outline = COGL_INVALID_HANDLE;
+      static CoglPipeline *outline = NULL;
       guint8 color_intensity;
       int i;
+      CoglVertexAttribute *loop_attributes[2];
 
       _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
 
-      if (outline == COGL_INVALID_HANDLE)
-        outline = cogl_material_new ();
+      if (outline == NULL)
+        outline = cogl_pipeline_new ();
 
       /* The least significant three bits represent the three
          components so that the order of colours goes red, green,
@@ -256,7 +258,7 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
          of 24 colours. If there are more than 24 batches on the stage
          then it will wrap around */
       color_intensity = 0xff - 0x33 * (ctxt->journal_rectangles_color >> 3);
-      cogl_material_set_color4ub (outline,
+      cogl_pipeline_set_color4ub (outline,
                                   (ctxt->journal_rectangles_color & 1) ?
                                   color_intensity : 0,
                                   (ctxt->journal_rectangles_color & 2) ?
@@ -264,10 +266,14 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
                                   (ctxt->journal_rectangles_color & 4) ?
                                   color_intensity : 0,
                                   0xff);
-      _cogl_material_flush_gl_state (outline, FALSE);
-      _cogl_enable (COGL_ENABLE_VERTEX_ARRAY);
+      cogl_set_source (outline);
+
+      loop_attributes[0] = attributes[0]; /* we just want the position */
+      loop_attributes[1] = NULL;
       for (i = 0; i < batch_len; i++)
-        GE( glDrawArrays (GL_LINE_LOOP, 4 * i + state->vertex_offset, 4) );
+        _cogl_draw_vertex_attributes_array (COGL_VERTICES_MODE_LINE_LOOP,
+                                            4 * i + state->current_vertex, 4,
+                                            loop_attributes);
 
       /* Go to the next color */
       do
@@ -278,7 +284,11 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
              || (ctxt->journal_rectangles_color & 0x07) == 0x07);
     }
 
-  state->vertex_offset += (4 * batch_len);
+  state->current_vertex += (4 * batch_len);
+
+  cogl_pop_source ();
+
+  COGL_TIMER_STOP (_cogl_uprof_context, time_flush_modelview_and_entries);
 }
 
 static gboolean
@@ -304,28 +314,27 @@ compare_entry_modelviews (CoglJournalEntry *entry0,
 }
 
 /* At this point we have a run of quads that we know have compatible
- * materials, but they may not all have the same modelview matrix */
+ * pipelines, but they may not all have the same modelview matrix */
 static void
-_cogl_journal_flush_material_and_entries (CoglJournalEntry *batch_start,
+_cogl_journal_flush_pipeline_and_entries (CoglJournalEntry *batch_start,
                                           int               batch_len,
                                           void             *data)
 {
-  unsigned long enable_flags = 0;
+  CoglJournalFlushState *state = data;
+  COGL_STATIC_TIMER (time_flush_pipeline_entries,
+                     "flush: texcoords+pipeline+entries", /* parent */
+                     "flush: pipeline+entries",
+                     "The time spent flushing pipeline + entries",
+                     0 /* no application private data */);
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  COGL_TIMER_START (_cogl_uprof_context, time_flush_pipeline_entries);
+
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_BATCHING))
-    g_print ("BATCHING:   material batch len = %d\n", batch_len);
+    g_print ("BATCHING:    pipeline batch len = %d\n", batch_len);
 
-  _cogl_material_flush_gl_state (batch_start->material, TRUE);
-
-  if (ctx->enable_backface_culling)
-    enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
-
-  enable_flags |= COGL_ENABLE_VERTEX_ARRAY;
-  enable_flags |= COGL_ENABLE_COLOR_ARRAY;
-  _cogl_enable (enable_flags);
-  _cogl_flush_face_winding ();
+  state->source = batch_start->pipeline;
 
   /* If we haven't transformed the quads in software then we need to also break
    * up batches according to changes in the modelview matrix... */
@@ -339,19 +348,21 @@ _cogl_journal_flush_material_and_entries (CoglJournalEntry *batch_start,
     }
   else
     _cogl_journal_flush_modelview_and_entries (batch_start, batch_len, data);
+
+  COGL_TIMER_STOP (_cogl_uprof_context, time_flush_pipeline_entries);
 }
 
 static gboolean
-compare_entry_materials (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
+compare_entry_pipelines (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
 {
-  /* batch rectangles using compatible materials */
+  /* batch rectangles using compatible pipelines */
 
-  /* XXX: _cogl_material_equal may give false negatives since it avoids
+  /* XXX: _cogl_pipeline_equal may give false negatives since it avoids
    * deep comparisons as an optimization. It aims to compare enough so
    * that we that we are able to batch the 90% common cases, but may not
    * look at less common differences. */
-  if (_cogl_material_equal (entry0->material,
-                            entry1->material,
+  if (_cogl_pipeline_equal (entry0->pipeline,
+                            entry1->pipeline,
                             TRUE))
     return TRUE;
   else
@@ -369,37 +380,75 @@ _cogl_journal_flush_texcoord_vbo_offsets_and_entries (
 {
   CoglJournalFlushState *state = data;
   int                    i;
+  COGL_STATIC_TIMER (time_flush_texcoord_pipeline_entries,
+                     "flush: vbo+texcoords+pipeline+entries", /* parent */
+                     "flush: texcoords+pipeline+entries",
+                     "The time spent flushing texcoord offsets + pipeline "
+                     "+ entries",
+                     0 /* no application private data */);
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  COGL_TIMER_START (_cogl_uprof_context, time_flush_texcoord_pipeline_entries);
+
+  /* NB: attributes 0 and 1 are position and color */
+
+  for (i = 2; i < state->attributes->len; i++)
+    cogl_object_unref (g_array_index (state->attributes,
+                                      CoglVertexAttribute *, i));
+
+  g_array_set_size (state->attributes, batch_start->n_layers + 2);
+
   for (i = 0; i < batch_start->n_layers; i++)
     {
-      GE (glClientActiveTexture (GL_TEXTURE0 + i));
-      GE (glEnableClientState (GL_TEXTURE_COORD_ARRAY));
+      CoglVertexAttribute **attribute_entry =
+        &g_array_index (state->attributes, CoglVertexAttribute *, i + 2);
+      const char *names[] = {
+          "cogl_tex_coord0_in",
+          "cogl_tex_coord1_in",
+          "cogl_tex_coord2_in",
+          "cogl_tex_coord3_in",
+          "cogl_tex_coord4_in",
+          "cogl_tex_coord5_in",
+          "cogl_tex_coord6_in",
+          "cogl_tex_coord7_in"
+      };
+      char *name;
+
       /* XXX NB:
        * Our journal's vertex data is arranged as follows:
        * 4 vertices per quad:
-       *    2 or 3 GLfloats per position (3 when doing software transforms)
-       *    4 RGBA GLubytes,
-       *    2 GLfloats per tex coord * n_layers
+       *    2 or 3 floats per position (3 when doing software transforms)
+       *    4 RGBA bytes,
+       *    2 floats per tex coord * n_layers
        * (though n_layers may be padded; see definition of
        *  GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS for details)
        */
-      GE (glTexCoordPointer (2, GL_FLOAT, state->stride,
-                             (void *)(state->vbo_offset +
-                                      (POS_STRIDE + COLOR_STRIDE) * 4 +
-                                      TEX_STRIDE * 4 * i)));
-    }
+      name = i < 8 ? (char *)names[i] :
+        g_strdup_printf ("cogl_tex_coord%d_in", i);
 
-  _cogl_bitmask_clear_all (&ctx->temp_bitmask);
-  _cogl_bitmask_set_range (&ctx->temp_bitmask, batch_start->n_layers, TRUE);
-  _cogl_disable_other_texcoord_arrays (&ctx->temp_bitmask);
+      /* XXX: it may be worth having some form of static initializer for
+       * attributes... */
+      *attribute_entry =
+        cogl_vertex_attribute_new (state->vertex_array,
+                                   name,
+                                   state->stride,
+                                   state->array_offset +
+                                      (POS_STRIDE + COLOR_STRIDE) * 4 +
+                                      TEX_STRIDE * 4 * i,
+                                   2,
+                                   COGL_VERTEX_ATTRIBUTE_TYPE_FLOAT);
+
+      if (i >= 8)
+        g_free (name);
+    }
 
   batch_and_call (batch_start,
                   batch_len,
-                  compare_entry_materials,
-                  _cogl_journal_flush_material_and_entries,
+                  compare_entry_pipelines,
+                  _cogl_journal_flush_pipeline_and_entries,
                   data);
+  COGL_TIMER_STOP (_cogl_uprof_context, time_flush_texcoord_pipeline_entries);
 }
 
 static gboolean
@@ -419,17 +468,23 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
                                              void             *data)
 {
   CoglJournalFlushState   *state = data;
-  gsize                   stride;
-#ifndef HAVE_COGL_GL
-  int                      needed_indices = batch_len * 6;
-  CoglHandle               indices_handle;
-  CoglVertexBufferIndices *indices;
-#endif
+  gsize                    stride;
+  int                      i;
+  CoglVertexAttribute    **attribute_entry;
+  COGL_STATIC_TIMER (time_flush_vbo_texcoord_pipeline_entries,
+                     "flush: clip+vbo+texcoords+pipeline+entries", /* parent */
+                     "flush: vbo+texcoords+pipeline+entries",
+                     "The time spent flushing vbo + texcoord offsets + "
+                     "pipeline + entries",
+                     0 /* no application private data */);
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  COGL_TIMER_START (_cogl_uprof_context,
+                    time_flush_vbo_texcoord_pipeline_entries);
+
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_BATCHING))
-    g_print ("BATCHING:  vbo offset batch len = %d\n", batch_len);
+    g_print ("BATCHING:   vbo offset batch len = %d\n", batch_len);
 
   /* XXX NB:
    * Our journal's vertex data is arranged as follows:
@@ -441,35 +496,45 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
    *  GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS for details)
    */
   stride = GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS (batch_start->n_layers);
-  stride *= sizeof (GLfloat);
+  stride *= sizeof (float);
   state->stride = stride;
 
-  GE (glVertexPointer (N_POS_COMPONENTS, GL_FLOAT, stride,
-                       (void *)state->vbo_offset));
-  GE (glColorPointer (4, GL_UNSIGNED_BYTE, stride,
-                      (void *)(state->vbo_offset + (POS_STRIDE * 4))));
+  for (i = 0; i < state->attributes->len; i++)
+    cogl_object_unref (g_array_index (state->attributes,
+                                      CoglVertexAttribute *, i));
+
+  g_array_set_size (state->attributes, 2);
+
+  attribute_entry =
+    &g_array_index (state->attributes, CoglVertexAttribute *, 0);
+  *attribute_entry =
+    cogl_vertex_attribute_new (state->vertex_array,
+                               "cogl_position_in",
+                               stride,
+                               state->array_offset,
+                               N_POS_COMPONENTS,
+                               COGL_VERTEX_ATTRIBUTE_TYPE_FLOAT);
+
+  attribute_entry =
+    &g_array_index (state->attributes, CoglVertexAttribute *, 1);
+  *attribute_entry =
+    cogl_vertex_attribute_new (state->vertex_array,
+                               "cogl_color_in",
+                               stride,
+                               state->array_offset + (POS_STRIDE * 4),
+                               4,
+                               COGL_VERTEX_ATTRIBUTE_TYPE_UNSIGNED_BYTE);
 
 #ifndef HAVE_COGL_GL
-  indices_handle = cogl_vertex_buffer_indices_get_for_quads (needed_indices);
-  indices = _cogl_vertex_buffer_indices_pointer_from_handle (indices_handle);
-  state->indices = indices;
-
-  if (indices->type == GL_UNSIGNED_BYTE)
-    state->indices_type_size = 1;
-  else if (indices->type == GL_UNSIGNED_SHORT)
-    state->indices_type_size = 2;
-  else
-    g_critical ("unknown indices type %d", indices->type);
-
-  GE (glBindBuffer (GL_ELEMENT_ARRAY_BUFFER,
-                    GPOINTER_TO_UINT (indices->vbo_name)));
+  state->indices = cogl_get_rectangle_indices (batch_len);
 #endif
 
-  /* We only call gl{Vertex,Color,Texture}Pointer when the stride within
-   * the VBO changes. (due to a change in the number of material layers)
+  /* We only create new VertexAttributes when the stride within the
+   * VertexArray changes. (due to a change in the number of pipeline layers)
    * While the stride remains constant we walk forward through the above
-   * VBO using a vertex offset passed to glDraw{Arrays,Elements} */
-  state->vertex_offset = 0;
+   * VertexArray using a vertex offset passed to cogl_draw_vertex_attributes
+   */
+  state->current_vertex = 0;
 
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_JOURNAL))
     {
@@ -477,9 +542,9 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
 
       if (cogl_get_features () & COGL_FEATURE_VBOS)
         verts = ((guint8 *)ctx->logged_vertices->data) +
-          (size_t)state->vbo_offset;
+          (size_t)state->array_offset;
       else
-        verts = (guint8 *)state->vbo_offset;
+        verts = (guint8 *)state->array_offset;
       _cogl_journal_dump_quad_batch (verts,
                                      batch_start->n_layers,
                                      batch_len);
@@ -492,16 +557,19 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
                   data);
 
   /* progress forward through the VBO containing all our vertices */
-  state->vbo_offset += (stride * 4 * batch_len);
+  state->array_offset += (stride * 4 * batch_len);
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_JOURNAL))
-    g_print ("new vbo offset = %lu\n", (unsigned long)state->vbo_offset);
+    g_print ("new vbo offset = %lu\n", (unsigned long)state->array_offset);
+
+  COGL_TIMER_STOP (_cogl_uprof_context,
+                   time_flush_vbo_texcoord_pipeline_entries);
 }
 
 static gboolean
 compare_entry_strides (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
 {
   /* Currently the only thing that affects the stride for our vertex arrays
-   * is the number of material layers. We need to update our VBO offsets
+   * is the number of pipeline layers. We need to update our VBO offsets
    * whenever the stride changes. */
   /* TODO: We should be padding the n_layers == 1 case as if it were
    * n_layers == 2 so we can reduce the need to split batches. */
@@ -513,33 +581,94 @@ compare_entry_strides (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
     return FALSE;
 }
 
-static GLuint
-upload_vertices_to_vbo (GArray *vertices, CoglJournalFlushState *state)
+/* At this point we know the batch has a unique clip stack */
+static void
+_cogl_journal_flush_clip_stacks_and_entries (CoglJournalEntry *batch_start,
+                                             int               batch_len,
+                                             void             *data)
+{
+  CoglJournalFlushState *state = data;
+
+  COGL_STATIC_TIMER (time_flush_clip_stack_pipeline_entries,
+                     "Journal Flush", /* parent */
+                     "flush: clip+vbo+texcoords+pipeline+entries",
+                     "The time spent flushing clip + vbo + texcoord offsets + "
+                     "pipeline + entries",
+                     0 /* no application private data */);
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  COGL_TIMER_START (_cogl_uprof_context,
+                    time_flush_clip_stack_pipeline_entries);
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_BATCHING))
+    g_print ("BATCHING:  clip stack batch len = %d\n", batch_len);
+
+  _cogl_clip_stack_flush (batch_start->clip_stack);
+
+  _cogl_matrix_stack_push (state->modelview_stack);
+
+  /* If we have transformed all our quads at log time then we ensure
+   * no further model transform is applied by loading the identity
+   * matrix here. We need to do this after flushing the clip stack
+   * because the clip stack flushing code can modify the matrix */
+  if (G_LIKELY (!(cogl_debug_flags & COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM)))
+    {
+      _cogl_matrix_stack_load_identity (state->modelview_stack);
+      _cogl_matrix_stack_flush_to_gl (state->modelview_stack,
+                                      COGL_MATRIX_MODELVIEW);
+    }
+
+  /* Setting up the clip state can sometimes also flush the projection
+     matrix so we should flush it again. This will be a no-op if the
+     clip code didn't modify the projection */
+  _cogl_matrix_stack_flush_to_gl (state->projection_stack,
+                                  COGL_MATRIX_PROJECTION);
+
+  batch_and_call (batch_start,
+                  batch_len,
+                  compare_entry_strides,
+                  _cogl_journal_flush_vbo_offsets_and_entries, /* callback */
+                  data);
+
+  _cogl_matrix_stack_pop (state->modelview_stack);
+
+  COGL_TIMER_STOP (_cogl_uprof_context,
+                   time_flush_clip_stack_pipeline_entries);
+}
+
+static gboolean
+compare_entry_clip_stacks (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
+{
+  return entry0->clip_stack == entry1->clip_stack;
+}
+
+static CoglVertexArray *
+upload_vertices (GArray *vertices, CoglJournalFlushState *state)
 {
   gsize needed_vbo_len;
-  GLuint journal_vbo;
+  CoglVertexArray *array;
+  CoglBuffer *buffer;
 
   _COGL_GET_CONTEXT (ctx, 0);
 
-  needed_vbo_len = vertices->len * sizeof (GLfloat);
-
+  needed_vbo_len = vertices->len * sizeof (float);
   g_assert (needed_vbo_len);
-  GE (glGenBuffers (1, &journal_vbo));
-  GE (glBindBuffer (GL_ARRAY_BUFFER, journal_vbo));
-  GE (glBufferData (GL_ARRAY_BUFFER,
-                    needed_vbo_len,
-                    vertices->data,
-                    GL_STATIC_DRAW));
+
+  array = cogl_vertex_array_new (needed_vbo_len, NULL);
+  buffer = COGL_BUFFER (array);
+  cogl_buffer_set_update_hint (buffer, COGL_BUFFER_UPDATE_HINT_STATIC);
+  cogl_buffer_set_data (buffer, 0, vertices->data, needed_vbo_len);
 
   /* As we flush the journal entries in batches we walk forward through the
    * above VBO starting at offset 0... */
-  state->vbo_offset = NULL;
+  state->array_offset = 0;
 
-  return journal_vbo;
+  return array;
 }
 
 /* XXX NB: When _cogl_journal_flush() returns all state relating
- * to materials, all glEnable flags and current matrix state
+ * to pipelines, all glEnable flags and current matrix state
  * is undefined.
  */
 void
@@ -547,9 +676,6 @@ _cogl_journal_flush (void)
 {
   CoglJournalFlushState state;
   int                   i;
-  GLuint                journal_vbo;
-  gboolean              vbo_fallback =
-    (cogl_get_features () & COGL_FEATURE_VBOS) ? FALSE : TRUE;
   CoglFramebuffer      *framebuffer;
   CoglMatrixStack      *modelview_stack;
   COGL_STATIC_TIMER (flush_timer,
@@ -568,68 +694,58 @@ _cogl_journal_flush (void)
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_BATCHING))
     g_print ("BATCHING: journal len = %d\n", ctx->journal->len);
 
-  /* Load all the vertex data we have accumulated so far into a single VBO
-   * to minimize memory management costs within the GL driver. */
-  if (!vbo_fallback)
-    journal_vbo = upload_vertices_to_vbo (ctx->logged_vertices, &state);
-  else
-    state.vbo_offset = (char *)ctx->logged_vertices->data;
+  state.vertex_array = upload_vertices (ctx->logged_vertices, &state);
+  state.attributes = ctx->journal_flush_attributes_array;
 
   framebuffer = _cogl_get_framebuffer ();
   modelview_stack = _cogl_framebuffer_get_modelview_stack (framebuffer);
   state.modelview_stack = modelview_stack;
-
-  _cogl_matrix_stack_push (modelview_stack);
-
-  /* If we have transformed all our quads at log time then we ensure no
-   * further model transform is applied by loading the identity matrix
-   * here... */
-  if (G_LIKELY (!(cogl_debug_flags & COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM)))
-    {
-      _cogl_matrix_stack_load_identity (modelview_stack);
-      _cogl_matrix_stack_flush_to_gl (modelview_stack, COGL_MATRIX_MODELVIEW);
-    }
+  state.projection_stack = _cogl_framebuffer_get_projection_stack (framebuffer);
 
   /* batch_and_call() batches a list of journal entries according to some
    * given criteria and calls a callback once for each determined batch.
    *
    * The process of flushing the journal is staggered to reduce the amount
    * of driver/GPU state changes necessary:
-   * 1) We split the entries according to the stride of the vertices:
+   * 1) We split the entries according to the clip state.
+   * 2) We split the entries according to the stride of the vertices:
    *      Each time the stride of our vertex data changes we need to call
    *      gl{Vertex,Color}Pointer to inform GL of new VBO offsets.
    *      Currently the only thing that affects the stride of our vertex data
-   *      is the number of material layers.
-   * 2) We split the entries explicitly by the number of material layers:
+   *      is the number of pipeline layers.
+   * 3) We split the entries explicitly by the number of pipeline layers:
    *      We pad our vertex data when the number of layers is < 2 so that we
    *      can minimize changes in stride. Each time the number of layers
    *      changes we need to call glTexCoordPointer to inform GL of new VBO
    *      offsets.
-   * 3) We then split according to compatible Cogl materials:
-   *      This is where we flush material state
-   * 4) Finally we split according to modelview matrix changes:
+   * 4) We then split according to compatible Cogl pipelines:
+   *      This is where we flush pipeline state
+   * 5) Finally we split according to modelview matrix changes:
    *      This is when we finally tell GL to draw something.
    *      Note: Splitting by modelview changes is skipped when are doing the
    *      vertex transformation in software at log time.
    */
   batch_and_call ((CoglJournalEntry *)ctx->journal->data, /* first entry */
                   ctx->journal->len, /* max number of entries to consider */
-                  compare_entry_strides,
-                  _cogl_journal_flush_vbo_offsets_and_entries, /* callback */
+                  compare_entry_clip_stacks,
+                  _cogl_journal_flush_clip_stacks_and_entries, /* callback */
                   &state); /* data */
 
-  _cogl_matrix_stack_pop (modelview_stack);
+  for (i = 0; i < state.attributes->len; i++)
+    cogl_object_unref (g_array_index (state.attributes,
+                                      CoglVertexAttribute *, i));
+  g_array_set_size (state.attributes, 0);
+
+  cogl_object_unref (state.vertex_array);
 
   for (i = 0; i < ctx->journal->len; i++)
     {
       CoglJournalEntry *entry =
         &g_array_index (ctx->journal, CoglJournalEntry, i);
-      _cogl_material_journal_unref (entry->material);
+      _cogl_pipeline_journal_unref (entry->pipeline);
+      _cogl_clip_stack_unref (entry->clip_stack);
     }
 
-  if (!vbo_fallback)
-    GE (glDeleteBuffers (1, &journal_vbo));
-
   g_array_set_size (ctx->journal, 0);
   g_array_set_size (ctx->logged_vertices, 0);
 
@@ -643,19 +759,18 @@ _cogl_journal_init (void)
    * next the the journal is flushed. Note: This lets up flush things
    * that themselves depend on the journal, such as clip state. */
 
-  /* NB: the journal deals with flushing the modelview stack manually */
+  /* NB: the journal deals with flushing the modelview stack and clip
+     state manually */
   _cogl_framebuffer_flush_state (_cogl_get_framebuffer (),
-                                 COGL_FRAMEBUFFER_FLUSH_SKIP_MODELVIEW);
+                                 COGL_FRAMEBUFFER_FLUSH_SKIP_MODELVIEW |
+                                 COGL_FRAMEBUFFER_FLUSH_SKIP_CLIP_STATE);
 }
 
 void
 _cogl_journal_log_quad (const float  *position,
-                        CoglHandle    material,
+                        CoglPipeline *pipeline,
                         int           n_layers,
-                        guint32       fallback_layers,
-                        GLuint        layer0_override_texture,
-                        const CoglMaterialWrapModeOverrides *
-                                      wrap_mode_overrides,
+                        CoglHandle    layer0_override_texture,
                         const float  *tex_coords,
                         unsigned int  tex_coords_len)
 {
@@ -669,8 +784,8 @@ _cogl_journal_log_quad (const float  *position,
   int               next_entry;
   guint32           disable_layers;
   CoglJournalEntry *entry;
-  CoglHandle        source;
-  CoglMaterialFlushOptions flush_options;
+  CoglPipeline     *source;
+  CoglPipelineFlushOptions flush_options;
   COGL_STATIC_TIMER (log_timer,
                      "Mainloop", /* parent */
                      "Journal Log",
@@ -707,7 +822,7 @@ _cogl_journal_log_quad (const float  *position,
 
   /* FIXME: This is a hacky optimization, since it will break if we
    * change the definition of CoglColor: */
-  _cogl_material_get_colorubv (material, c);
+  _cogl_pipeline_get_colorubv (pipeline, c);
   src_c = c;
   for (i = 0; i < 3; i++)
     {
@@ -789,49 +904,40 @@ _cogl_journal_log_quad (const float  *position,
 
   entry->n_layers = n_layers;
 
-  source = material;
+  source = pipeline;
 
   if (G_UNLIKELY (ctx->legacy_state_set))
     {
-      source = cogl_material_copy (material);
-      _cogl_material_apply_legacy_state (source);
+      source = cogl_pipeline_copy (pipeline);
+      _cogl_pipeline_apply_legacy_state (source);
     }
 
   flush_options.flags = 0;
-  if (G_UNLIKELY (cogl_material_get_n_layers (material) != n_layers))
+  if (G_UNLIKELY (cogl_pipeline_get_n_layers (pipeline) != n_layers))
     {
       disable_layers = (1 << n_layers) - 1;
       disable_layers = ~disable_layers;
       flush_options.disable_layers = disable_layers;
-      flush_options.flags |= COGL_MATERIAL_FLUSH_DISABLE_MASK;
-    }
-  if (G_UNLIKELY (fallback_layers))
-    {
-      flush_options.fallback_layers = fallback_layers;
-      flush_options.flags |= COGL_MATERIAL_FLUSH_FALLBACK_MASK;
+      flush_options.flags |= COGL_PIPELINE_FLUSH_DISABLE_MASK;
     }
   if (G_UNLIKELY (layer0_override_texture))
     {
-      flush_options.flags |= COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE;
+      flush_options.flags |= COGL_PIPELINE_FLUSH_LAYER0_OVERRIDE;
       flush_options.layer0_override_texture = layer0_override_texture;
     }
-  if (wrap_mode_overrides)
-    {
-      flush_options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
-      flush_options.wrap_mode_overrides = *wrap_mode_overrides;
-    }
 
   if (G_UNLIKELY (flush_options.flags))
     {
-      /* If we haven't already created a derived material... */
-      if (source == material)
-        source = cogl_material_copy (material);
-      _cogl_material_apply_overrides (source, &flush_options);
+      /* If we haven't already created a derived pipeline... */
+      if (source == pipeline)
+        source = cogl_pipeline_copy (pipeline);
+      _cogl_pipeline_apply_overrides (source, &flush_options);
     }
 
-  entry->material = _cogl_material_journal_ref (source);
+  entry->pipeline = _cogl_pipeline_journal_ref (source);
+  entry->clip_stack = _cogl_clip_stack_ref (_cogl_get_clip_stack ());
 
-  if (G_UNLIKELY (source != material))
+  if (G_UNLIKELY (source != pipeline))
     cogl_handle_unref (source);
 
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM))
