@@ -26,7 +26,7 @@
 
 #include "cogl-bitmap-private.h"
 #include "cogl-handle.h"
-#include "cogl-material-private.h"
+#include "cogl-pipeline-private.h"
 
 #define COGL_TEXTURE(tex) ((CoglTexture *)(tex))
 
@@ -34,8 +34,6 @@ typedef struct _CoglTexture           CoglTexture;
 typedef struct _CoglTextureVtable     CoglTextureVtable;
 
 typedef void (*CoglTextureSliceCallback) (CoglHandle handle,
-                                          GLuint gl_handle,
-                                          GLenum gl_target,
                                           const float *slice_coords,
                                           const float *virtual_coords,
                                           void *user_data);
@@ -145,7 +143,7 @@ typedef enum _CoglTextureChangeFlags
   /* Whenever the internals of a texture are changed such that the
    * underlying GL textures that represent the CoglTexture change then
    * we notify cogl-material.c via
-   * _cogl_material_texture_pre_change_notify
+   * _cogl_pipeline_texture_pre_change_notify
    */
   COGL_TEXTURE_CHANGE_GL_TEXTURES
 
