@@ -40,19 +40,22 @@
 #include "clutter-stage-egl.h"
 #include "clutter-egl.h"
 
-#include "../clutter-private.h"
-#include "../clutter-main.h"
-#include "../clutter-debug.h"
-#include "../clutter-version.h"
+#include "clutter-debug.h"
+#include "clutter-private.h"
+#include "clutter-main.h"
+#include "clutter-stage-private.h"
+#include "clutter-version.h"
 
 static ClutterBackendEGL *backend_singleton = NULL;
 
 static const gchar *clutter_fb_device = NULL;
 
+static gchar *clutter_vblank_name = NULL;
+
 #ifdef COGL_HAS_X11_SUPPORT
-G_DEFINE_TYPE (ClutterBackendEGL, clutter_backend_egl, CLUTTER_TYPE_BACKEND_X11);
+G_DEFINE_TYPE (ClutterBackendEGL, _clutter_backend_egl, CLUTTER_TYPE_BACKEND_X11);
 #else
-G_DEFINE_TYPE (ClutterBackendEGL, clutter_backend_egl, CLUTTER_TYPE_BACKEND);
+G_DEFINE_TYPE (ClutterBackendEGL, _clutter_backend_egl, CLUTTER_TYPE_BACKEND);
 #endif
 
 static void
@@ -69,12 +72,19 @@ clutter_backend_egl_pre_parse (ClutterBackend  *backend,
   const gchar *env_string;
 #ifdef COGL_HAS_X11_SUPPORT
   ClutterBackendClass *backend_x11_class =
-    CLUTTER_BACKEND_CLASS (clutter_backend_egl_parent_class);
+    CLUTTER_BACKEND_CLASS (_clutter_backend_egl_parent_class);
 
   if (!backend_x11_class->pre_parse (backend, error))
     return FALSE;
 #endif
 
+  env_string = g_getenv ("CLUTTER_VBLANK");
+  if (env_string)
+    {
+      clutter_vblank_name = g_strdup (env_string);
+      env_string = NULL;
+    }
+
   env_string = g_getenv ("CLUTTER_FB_DEVICE");
   if (env_string != NULL && env_string[0] != '\0')
     clutter_fb_device = g_strdup (env_string);
@@ -90,7 +100,7 @@ clutter_backend_egl_post_parse (ClutterBackend  *backend,
 #ifdef COGL_HAS_X11_SUPPORT
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
   ClutterBackendClass *backend_x11_class =
-    CLUTTER_BACKEND_CLASS (clutter_backend_egl_parent_class);
+    CLUTTER_BACKEND_CLASS (_clutter_backend_egl_parent_class);
 #endif
   EGLBoolean status;
 
@@ -132,6 +142,34 @@ clutter_backend_egl_post_parse (ClutterBackend  *backend,
   return TRUE;
 }
 
+void
+_clutter_backend_egl_blit_sub_buffer (ClutterBackendEGL *backend_egl,
+                                      EGLSurface drawable,
+                                      int x, int y, int width, int height)
+{
+  if (backend_egl->swap_buffers_region)
+    {
+      EGLint rect[4] = {
+          x, y, width, height
+      };
+      backend_egl->swap_buffers_region (backend_egl->edpy,
+                                        drawable,
+                                        1,
+                                        rect);
+    }
+#if 0 /* XXX need GL_ARB_draw_buffers */
+  else if (backend_egl->blit_framebuffer)
+    {
+      glDrawBuffer (GL_FRONT);
+      backend_egl->blit_framebuffer (x, y, x + width, y + height,
+                                     x, y, x + width, y + height,
+                                     GL_COLOR_BUFFER_BIT, GL_NEAREST);
+      glDrawBuffer (GL_BACK);
+      glFlush();
+    }
+#endif
+}
+
 static gboolean
 clutter_backend_egl_create_context (ClutterBackend  *backend,
                                     GError         **error)
@@ -475,7 +513,7 @@ clutter_backend_egl_redraw (ClutterBackend *backend,
 
   g_assert (CLUTTER_IS_STAGE_EGL (impl));
 
-  clutter_stage_egl_redraw (CLUTTER_STAGE_EGL (impl), stage);
+  _clutter_stage_egl_redraw (CLUTTER_STAGE_EGL (impl), stage);
 }
 
 #ifdef HAVE_TSLIB
@@ -493,7 +531,7 @@ clutter_backend_egl_finalize (GObject *gobject)
   if (backend_singleton)
     backend_singleton = NULL;
 
-  G_OBJECT_CLASS (clutter_backend_egl_parent_class)->finalize (gobject);
+  G_OBJECT_CLASS (_clutter_backend_egl_parent_class)->finalize (gobject);
 }
 
 static void
@@ -510,7 +548,7 @@ clutter_backend_egl_dispose (GObject *gobject)
      ClutterBackendX11 will destroy all of the stages before we
      destroy the egl context. Otherwise the actors may try to make GL
      calls during destruction which causes a crash */
-  G_OBJECT_CLASS (clutter_backend_egl_parent_class)->dispose (gobject);
+  G_OBJECT_CLASS (_clutter_backend_egl_parent_class)->dispose (gobject);
 
 #ifdef HAVE_TSLIB
   /* XXX: This should be renamed to _clutter_events_tslib_uninit */
@@ -583,7 +621,7 @@ clutter_backend_egl_constructor (GType                  gtype,
 
   if (!backend_singleton)
     {
-      parent_class = G_OBJECT_CLASS (clutter_backend_egl_parent_class);
+      parent_class = G_OBJECT_CLASS (_clutter_backend_egl_parent_class);
       retval = parent_class->constructor (gtype, n_params, params);
 
       backend_singleton = CLUTTER_BACKEND_EGL (retval);
@@ -597,10 +635,21 @@ clutter_backend_egl_constructor (GType                  gtype,
   return g_object_ref (backend_singleton);
 }
 
+static gboolean
+check_vblank_env (const char *name)
+{
+  if (clutter_vblank_name && !g_ascii_strcasecmp (clutter_vblank_name, name))
+    return TRUE;
+
+  return FALSE;
+}
+
 static ClutterFeatureFlags
 clutter_backend_egl_get_features (ClutterBackend *backend)
 {
   ClutterBackendEGL  *backend_egl = CLUTTER_BACKEND_EGL (backend);
+  const gchar *egl_extensions = NULL;
+  const gchar *gl_extensions = NULL;
   ClutterFeatureFlags flags;
 
   g_assert (backend_egl->egl_context != NULL);
@@ -612,6 +661,39 @@ clutter_backend_egl_get_features (ClutterBackend *backend)
   flags = CLUTTER_FEATURE_STAGE_STATIC;
 #endif
 
+  /* First check for explicit disabling or it set elsewhere (eg NVIDIA) */
+  if (check_vblank_env ("none"))
+    backend_egl->vblank_type = CLUTTER_VBLANK_NONE;
+  else
+    backend_egl->vblank_type = CLUTTER_VBLANK_SWAP_INTERVAL;
+
+  egl_extensions = eglQueryString (backend_egl->edpy, EGL_EXTENSIONS);
+
+  if (_cogl_check_extension ("EGL_NOK_swap_region", egl_extensions))
+    {
+      CLUTTER_NOTE (BACKEND,
+                    "Using EGL_NOK_swap_region for sub_buffer copies");
+      backend_egl->swap_buffers_region =
+        (void *)cogl_get_proc_address ("eglSwapBuffersRegionNOK");
+      backend_egl->can_blit_sub_buffer = TRUE;
+      backend_egl->blit_sub_buffer_is_synchronized = TRUE;
+    }
+
+  gl_extensions = (const gchar *)glGetString (GL_EXTENSIONS);
+
+#if 0 /* XXX need GL_ARB_draw_buffers */
+  if (!backend_egl->swap_buffers_region &&
+      _cogl_check_extension ("GL_EXT_framebuffer_blit", gl_extensions))
+    {
+      CLUTTER_NOTE (BACKEND,
+                    "Using glBlitFramebuffer fallback for sub_buffer copies");
+      backend_egl->blit_framebuffer =
+        (BlitFramebufferProc) cogl_get_proc_address ("glBlitFramebuffer");
+      backend_egl->can_blit_sub_buffer = TRUE;
+      backend_egl->blit_sub_buffer_is_synchronized = FALSE;
+    }
+#endif
+
   CLUTTER_NOTE (BACKEND, "Checking features\n"
                 "GL_VENDOR: %s\n"
                 "GL_RENDERER: %s\n"
@@ -733,7 +815,7 @@ clutter_backend_egl_get_visual_info (ClutterBackendX11 *backend_x11)
 #endif
 
 static void
-clutter_backend_egl_class_init (ClutterBackendEGLClass *klass)
+_clutter_backend_egl_class_init (ClutterBackendEGLClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterBackendClass *backend_class = CLUTTER_BACKEND_CLASS (klass);
@@ -762,10 +844,9 @@ clutter_backend_egl_class_init (ClutterBackendEGLClass *klass)
 }
 
 static void
-clutter_backend_egl_init (ClutterBackendEGL *backend_egl)
+_clutter_backend_egl_init (ClutterBackendEGL *backend_egl)
 {
 #ifndef COGL_HAS_XLIB_SUPPORT
-  ClutterBackend *backend = CLUTTER_BACKEND (backend_egl);
 
 #ifdef HAVE_TSLIB
   backend_egl->event_timer = g_timer_new ();
@@ -781,11 +862,13 @@ clutter_backend_egl_init (ClutterBackendEGL *backend_egl)
 #endif
 }
 
+#ifdef CLUTTER_EGL_BACKEND_GENERIC
 GType
 _clutter_backend_impl_get_type (void)
 {
-  return clutter_backend_egl_get_type ();
+  return _clutter_backend_egl_get_type ();
 }
+#endif
 
 #ifdef COGL_HAS_XLIB_SUPPORT
 EGLDisplay
