@@ -46,9 +46,11 @@
 
 #include "xsettings/xsettings-common.h"
 
+#if HAVE_XCOMPOSITE
 #include <X11/extensions/Xcomposite.h>
+#endif
 
-#ifdef HAVE_XINPUT
+#if HAVE_XINPUT
 #include <X11/extensions/XInput.h>
 #endif
 
@@ -82,7 +84,7 @@ static const gchar *atom_names[] = {
   "UTF8_STRING",
 };
 
-static const guint n_atom_names = G_N_ELEMENTS (atom_names);
+#define N_ATOM_NAMES G_N_ELEMENTS (atom_names)
 
 /* singleton object */
 static ClutterBackendX11 *backend_singleton = NULL;
@@ -118,38 +120,6 @@ xsettings_filter (XEvent       *xevent,
   return CLUTTER_X11_FILTER_CONTINUE;
 }
 
-static Bool
-clutter_backend_x11_xsettings_watch (Window  window,
-                                     Bool    is_start,
-                                     long    mask,
-                                     void   *cb_data)
-{
-  ClutterBackendX11 *backend_x11 = cb_data;
-
-  if (is_start)
-    {
-      backend_x11->xsettings_xwin = window;
-      if (!backend_x11->xsettings_xwin)
-        return False;
-
-      clutter_x11_add_filter (xsettings_filter, backend_x11);
-
-      CLUTTER_NOTE (BACKEND, "Added filter on XSettings manager window 0x%x",
-                    (unsigned int) backend_x11->xsettings_xwin);
-    }
-  else
-    {
-      CLUTTER_NOTE (BACKEND, "Removed filter on XSettings manager window 0x%x",
-                    (unsigned int) backend_x11->xsettings_xwin);
-
-      clutter_x11_remove_filter (xsettings_filter, backend_x11);
-
-      backend_x11->xsettings_xwin = None;
-    }
-
-  return True;
-}
-
 static void
 clutter_backend_x11_xsettings_notify (const char       *name,
                                       XSettingsAction   action,
@@ -162,6 +132,9 @@ clutter_backend_x11_xsettings_notify (const char       *name,
   if (name == NULL || *name == '\0')
     return;
 
+  if (setting == NULL)
+    return;
+
   g_object_freeze_notify (G_OBJECT (settings));
 
   for (i = 0; i < _n_clutter_settings_map; i++)
@@ -287,7 +260,7 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
   if (backend_x11->xdpy)
     {
       ClutterSettings *settings;
-      Atom atoms[n_atom_names];
+      Atom atoms[N_ATOM_NAMES];
       double dpi;
 
       CLUTTER_NOTE (BACKEND, "Getting the X screen");
@@ -334,14 +307,16 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
         _clutter_xsettings_client_new (backend_x11->xdpy,
                                        backend_x11->xscreen_num,
                                        clutter_backend_x11_xsettings_notify,
-                                       clutter_backend_x11_xsettings_watch,
+                                       NULL,
                                        backend_x11);
 
+      clutter_x11_add_filter (xsettings_filter, backend_x11);
+
       if (clutter_synchronise)
         XSynchronize (backend_x11->xdpy, True);
 
       XInternAtoms (backend_x11->xdpy,
-                    (char **) atom_names, n_atom_names,
+                    (char **) atom_names, N_ATOM_NAMES,
                     False, atoms);
 
       backend_x11->atom_NET_WM_PID = atoms[0];
@@ -424,6 +399,7 @@ clutter_backend_x11_finalize (GObject *gobject)
 
   g_free (backend_x11->display_name);
 
+  clutter_x11_remove_filter (xsettings_filter, backend_x11);
   _clutter_xsettings_client_destroy (backend_x11->xsettings);
 
   XCloseDisplay (backend_x11->xdpy);
@@ -443,11 +419,7 @@ clutter_backend_x11_dispose (GObject *gobject)
   CLUTTER_NOTE (BACKEND, "Disposing the of stages");
   stage_manager = clutter_stage_manager_get_default ();
 
-  /* Destroy all of the stages. g_slist_foreach is used because the
-     finalizer for the stages will remove the stage from the
-     stage_manager's list and g_slist_foreach has some basic
-     protection against this */
-  g_slist_foreach (stage_manager->stages, (GFunc) clutter_actor_destroy, NULL);
+  g_object_unref (stage_manager);
 
   CLUTTER_NOTE (BACKEND, "Removing the event source");
   _clutter_backend_x11_events_uninit (CLUTTER_BACKEND (backend_x11));
@@ -940,6 +912,7 @@ clutter_x11_has_xinput (void)
 gboolean
 clutter_x11_has_composite_extension (void)
 {
+#if HAVE_XCOMPOSITE
   static gboolean have_composite = FALSE, done_check = FALSE;
   int error = 0, event = 0;
   Display *dpy;
@@ -968,6 +941,9 @@ clutter_x11_has_composite_extension (void)
   done_check = TRUE;
 
   return have_composite;
+#else
+  return FALSE;
+#endif /* HAVE_XCOMPOSITE */
 }
 
 /**
@@ -992,7 +968,7 @@ clutter_x11_has_composite_extension (void)
 void
 clutter_x11_set_use_argb_visual (gboolean use_argb)
 {
-  if (backend_singleton != NULL)
+  if (_clutter_context_is_initialized ())
     {
       g_warning ("%s() can only be used before calling clutter_init()",
                  G_STRFUNC);
